@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: ReadWriteLocker.h 2756 2007-12-11 10:57:59Z svn $
+ * $Id: ReadWriteLocker.h 2823 2008-03-15 15:05:34Z svn $
  *****************************************************************************/
 
 #ifndef READ_WRITE_LOCKER_H
@@ -44,6 +44,9 @@
 
 #include "AtomicWrapper.h"
 #include "apr_thread_proc.h"
+#ifdef DEBUG_ReadWriteLocker
+#include "apr_thread_mutex.h"
+#endif
 
 #include "Uncopyable.h"
 #include "Macro.h"
@@ -111,10 +114,17 @@ protected:
      *
      * @param[in] lock ロック変数
      */
-    void init(apr_atomic_t *lock, apr_uint32_t status)
+    void init(apr_atomic_t *lock, apr_uint32_t status
+#ifdef DEBUG_ReadWriteLocker
+              , apr_thread_mutex_t *mutex
+#endif
+              )
     {
         lock_ = lock;
         status_ = status;
+#ifdef DEBUG_ReadWriteLocker
+        mutex_ = mutex;
+#endif
     };
     /**
      * タイムアウト処理すべきかどうか判断します．
@@ -138,6 +148,21 @@ protected:
         return is_process_timeout_;
     };
 #endif
+    void log_try_count(const char *label,
+                       apr_size_t try_count, apr_uint32_t curr_status,
+                       apr_uint32_t new_status)
+    {
+#ifdef DEBUG_ReadWriteLocker
+#ifdef DEBUG_LOCK
+        apr_thread_mutex_lock(mutex_);
+        std::cerr << "try_count[" << label << "] = " << try_count << std::endl;
+        std::cerr << "state: " << std::hex << curr_status << " -> ";
+        std::cerr << std::hex << new_status << std::endl;
+        std::cerr.flush();
+        apr_thread_mutex_unlock(mutex_);
+#endif
+#endif
+    };
     /**
      * プロセッサーを明け渡します．
      */
@@ -168,25 +193,16 @@ protected:
     {
         return (status & WAIT_WRITE_MASK);
     };
-    static void log_try_count(const char *label, apr_size_t try_count,
-                              apr_uint32_t curr_status,
-                              apr_uint32_t new_status)
-    {
-#ifdef DEBUG_LOCK
-        std::cerr << "try_count[" << label << "] = " << try_count << std::endl;
-        std::cerr << "state: " << std::hex << curr_status << " -> ";
-        std::cerr << std::hex << new_status << std::endl;
-        std::cerr.flush();
-#endif
-    };
 
     // MEMO: テスト用
     friend void read_lock(void *data);
     friend void write_lock(void *data);
     template<class LockType> friend void lock_die(void *data1, void *data2);
-    friend void run_read(apr_pool_t *pool);
-    friend void run_write(apr_pool_t *pool);
-    friend void run_die(apr_pool_t *pool);
+    friend void run_read_or_write(const char *name, apr_size_t count,
+                                  void (* func)(void *data),
+                                  apr_pool_t *pool, apr_thread_mutex_t *mutex);
+    friend void run_read_and_write(apr_pool_t *pool, apr_thread_mutex_t *mutex);
+    friend void run_die(apr_pool_t *pool, apr_thread_mutex_t *mutex);
 
     /** 読み出し中のスレッドの数へのポインタ */
     apr_atomic_t *lock_;
@@ -194,6 +210,7 @@ protected:
     apr_time_t start_time_;
     apr_uint32_t status_;
 #ifdef DEBUG_ReadWriteLocker
+    apr_thread_mutex_t *mutex_;
     bool is_process_timeout_;
 #endif
 };
