@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (C) 2006 Tetsuya Kimata <kimata@acapulco.dyndns.org>
+ * Copyright (C) 2007 Tetsuya Kimata <kimata@acapulco.dyndns.org>
  *
  * All rights reserved.
  *
@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemIterator.h 2563 2007-08-28 04:12:55Z svn $
+ * $Id: UploadItemIterator.h 2756 2007-12-11 10:57:59Z svn $
  *****************************************************************************/
 
 #ifndef UPLOAD_ITEM_ITERATOR_H
@@ -58,7 +58,7 @@ public:
      */
     UploadItemIterator(apr_pool_t *pool, UploadItemManager *item_manager,
                        apr_size_t start_index, apr_size_t end_index,
-                       int(*cmp)(const void *a, const void *b));
+                       int (*cmp)(const UploadItem *a, const UploadItem *b));
     /**
      * コンストラクタです．
      *
@@ -105,7 +105,7 @@ public:
 private:
     void init(apr_pool_t *pool, UploadItemList *item_list,
               apr_size_t start_index, apr_size_t end_index,
-              int(*cmp)(const void *a, const void *b));
+              int (*cmp)(const UploadItem *a, const UploadItem *b));
     void init(apr_pool_t *pool, UploadItemList *item_list,
               apr_size_t start_index, apr_size_t end_index);
     void validate_index(UploadItemList *item_list,
