@@ -101,7 +101,10 @@ function updateProgress()
     if (progress.isFinish) {
         clearTimer();
         setProgresssInfo('Upload Succeeded!');
-        setInterval('location.reload(true)', 1000);
+        
+        setInterval('document.location = "' + uploaderURL + '/info/upload_id/' + uploadID + '"',
+                    1000);
+
         return;
     }
 
