@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: ReadWriteLocker.cpp 2758 2007-12-11 13:45:39Z svn $
+ * $Id: ReadWriteLocker.cpp 2824 2008-03-15 15:09:55Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -30,7 +30,7 @@
 #include "ReadWriteLocker.h"
 #include "SourceInfo.h"
 
-SOURCE_INFO_ADD("$Id: ReadWriteLocker.cpp 2758 2007-12-11 13:45:39Z svn $");
+SOURCE_INFO_ADD("$Id: ReadWriteLocker.cpp 2824 2008-03-15 15:09:55Z svn $");
 
 
 /******************************************************************************
@@ -75,17 +75,17 @@ bool ReadWriteLocker::should_timeout(apr_uint32_t status)
 #include "ReadLocker.h"
 #include "WriteLocker.h"
 
-const apr_size_t READ_COUNT             = 30;
-const apr_size_t WRITE_COUNT            = 3;
-const apr_size_t DIE_COUNT              = 3;
-const apr_size_t REPEAT_COUNT           = 100;
-const apr_size_t RETRY_COUNT            = 1000;
-const apr_size_t TIME_LIMIT_SEC         = 60;
+const apr_size_t READ_COUNT             = 1000;
+const apr_size_t WRITE_COUNT            = 100;
+const apr_size_t DIE_COUNT              = 100;
+const apr_size_t REPEAT_COUNT           = 10000;
+const apr_size_t TIME_LIMIT_SEC         = 120;
 
 typedef struct {
     apr_atomic_t lock;
     apr_atomic_t read_count;
     apr_atomic_t timeout_count;
+    apr_thread_mutex_t *mutex;
 } handle_t;
 
 typedef struct {
@@ -104,10 +104,9 @@ void read_lock(void *data)
 
     handle = reinterpret_cast<handle_t *>(data);
 
-    apr_sleep(rand() & 0xFF);
     ReadWriteLocker::yield();
 
-    ReadLocker read_lock(&(handle->lock));
+    ReadLocker read_lock(&(handle->lock), handle->mutex);
 
     apr_atomic_inc(&(handle->read_count));
 
@@ -120,7 +119,6 @@ void read_lock(void *data)
         THROW(MESSAGE_BUG_FOUND);
     }
 
-    apr_sleep(rand() & 0xFF);
     ReadWriteLocker::yield();
 
     apr_atomic_dec(&(handle->read_count));
@@ -133,9 +131,8 @@ void write_lock(void *data)
     handle = reinterpret_cast<handle_t *>(data);
 
     {
-        WriteLocker write_lock(&(handle->lock));
+        WriteLocker write_lock(&(handle->lock), handle->mutex);
 
-        apr_sleep(rand() & 0xFF);
         ReadWriteLocker::yield();
 
         if (write_lock.is_process_timeout()) {
@@ -151,7 +148,6 @@ void write_lock(void *data)
             THROW(MESSAGE_BUG_FOUND);
         }
 
-        apr_sleep((rand() & 0xFF) << 4);
         ReadWriteLocker::yield();
     }
 }
@@ -165,11 +161,10 @@ void lock_die(void *data1, void *data2)
     handle = reinterpret_cast<handle_t *>(data1);
     thread = reinterpret_cast<apr_thread_t *>(data2);
 
-    apr_sleep((rand() & 0xFF) << 4);
     ReadWriteLocker::yield();
 
     {
-        LockType lock(&(handle->lock));
+        LockType lock(&(handle->lock), handle->mutex);
         if (lock.is_process_timeout()) {
             apr_atomic_inc(&(handle->timeout_count));
         }
@@ -198,7 +193,7 @@ static void *APR_THREAD_FUNC worker(apr_thread_t *thread, void *data)
 }
 
 // NOTE: die_worker を friend 関数にするのは避けたかったので，
-// lock_die を使用している
+// lock_die を使用
 
 template<class LockType>
 static void *APR_THREAD_FUNC die_worker(apr_thread_t *thread,
@@ -209,34 +204,38 @@ static void *APR_THREAD_FUNC die_worker(apr_thread_t *thread,
     return NULL;
 }
 
-void run_read(apr_pool_t *pool)
+void run_read_or_write(const char *name, apr_size_t count,
+                       void (* func)(void *data),
+                       apr_pool_t *pool, apr_thread_mutex_t *mutex)
 {
     handle_t handle;
-    work_t read_work;
-    apr_thread_t *read_workers[READ_COUNT];
+    work_t work;
+    apr_thread_t **workers;
     apr_status_t status;
     bool is_ok;
     volatile double start_time;
     volatile double end_time;
 
-    show_test_name("read");
+    show_test_name(name);
 
     is_ok = true;
 
     handle.lock = 0;
     handle.read_count = 0;
     handle.timeout_count = 0;
+    handle.mutex = mutex;
 
-    read_work.handle = &handle;
-    read_work.func = read_lock;
+    work.handle = &handle;
+    work.func = func;
 
     start_time = get_time_sec();
 
-    for (apr_size_t i = 0; i < READ_COUNT; i++) {
-        apr_thread_create(read_workers + i, NULL, worker, &read_work, pool);
+    APR_PALLOC(workers, apr_thread_t **, pool, sizeof(apr_thread_t *)*count);
+    for (apr_size_t i = 0; i < count; i++) {
+        apr_thread_create(workers + i, NULL, worker, &work, pool);
     }
-    for (apr_size_t i = 0; i < READ_COUNT; i++) {
-        apr_thread_join(&status, read_workers[i]);
+    for (apr_size_t i = 0; i < count; i++) {
+        apr_thread_join(&status, workers[i]);
         if (status != APR_SUCCESS) {
             is_ok = false;
         }
@@ -256,10 +255,20 @@ void run_read(apr_pool_t *pool)
         THROW(MESSAGE_BUG_FOUND);
     }
 
-    show_item("read", (end_time - start_time)*1000, " msec");
+    show_item(name, (end_time - start_time)*1000, " msec");
 }
 
-void run_write(apr_pool_t *pool)
+void run_read(apr_pool_t *pool, apr_thread_mutex_t *mutex)
+{
+    return run_read_or_write("read", READ_COUNT, read_lock, pool, mutex);
+}
+
+void run_write(apr_pool_t *pool, apr_thread_mutex_t *mutex)
+{
+    return run_read_or_write("write", WRITE_COUNT, write_lock, pool, mutex);
+}
+
+void run_read_and_write(apr_pool_t *pool, apr_thread_mutex_t *mutex)
 {
     handle_t handle;
     work_t read_work;
@@ -272,7 +281,7 @@ void run_write(apr_pool_t *pool)
     volatile double start_time;
     volatile double end_time;
 
-    show_test_name("write");
+    show_test_name("read/write");
 
     is_read_ok = true;
     is_write_ok = true;
@@ -280,6 +289,7 @@ void run_write(apr_pool_t *pool)
     handle.lock = 0;
     handle.read_count = 0;
     handle.timeout_count = 0;
+    handle.mutex = mutex;
 
     read_work.handle = &handle;
     read_work.func = read_lock;
@@ -331,7 +341,7 @@ void run_write(apr_pool_t *pool)
     show_item("write", (end_time - start_time)*1000, " msec");
 }
 
-void run_die(apr_pool_t *pool)
+void run_die(apr_pool_t *pool, apr_thread_mutex_t *mutex)
 {
     handle_t handle;
     work_t read_work;
@@ -356,6 +366,7 @@ void run_die(apr_pool_t *pool)
     handle.lock = 0;
     handle.read_count = 0;
     handle.timeout_count = 0;
+    handle.mutex = mutex;
 
     read_work.handle = &handle;
     read_work.func = read_lock;
@@ -414,12 +425,11 @@ void run_die(apr_pool_t *pool)
         // たりする可能性がある．
         THROW(MESSAGE_BUG_FOUND);
     }
-    if (apr_atomic_read(&(handle.timeout_count)) < (DIE_COUNT + 1) ||
-        apr_atomic_read(&(handle.timeout_count)) > (DIE_COUNT * 2)) {
+    if (apr_atomic_read(&(handle.timeout_count)) > (DIE_COUNT * 2)) {
         // MEMO: Read と Write でそれぞれ DIE_COUNT 回ずつロックを保持
         // したまま終了させているが，Read に関してはネストしてロックで
         // きるので，タイムアウト回数の期待値としては DIE_COUNT + 1 以
-        // 上 DIE_COUNT * 2 以下となる．
+        // 上，DIE_COUNT * 2 以下となる．
         THROW(MESSAGE_BUG_FOUND);
     }
 
@@ -435,21 +445,27 @@ void run_die(apr_pool_t *pool)
 
 void run_all(apr_pool_t *pool, int argc, const char * const *argv)
 {
+    apr_thread_mutex_t *mutex;
+
     if (argc != 1) {
         THROW(MESSAGE_ARGUMENT_INVALID);
     }
 
     show_line();
 
+    apr_thread_mutex_create(&mutex, APR_THREAD_MUTEX_UNNESTED, pool);
     srand(static_cast<unsigned int>(apr_time_msec(apr_time_now())));
 
+
 #if APR_HAS_THREADS
-    run_read(pool);
-    run_write(pool);
-    run_die(pool);
+    run_read(pool, mutex);
+    run_write(pool, mutex);
+    run_read_and_write(pool, mutex);
+    run_die(pool, mutex);
 #else
     show_item("read", "skipped");
     show_item("write", "skipped");
+    show_item("read/write", "skipped");
     show_item("die", "skipped");
 #endif
 }
