@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItem.cpp 2762 2007-12-16 12:03:46Z svn $
+ * $Id: UploadItem.cpp 2818 2008-03-15 09:38:25Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -40,7 +40,7 @@
 #include "Auxiliary.h"
 #include "SourceInfo.h"
 
-SOURCE_INFO_ADD("$Id: UploadItem.cpp 2762 2007-12-16 12:03:46Z svn $");
+SOURCE_INFO_ADD("$Id: UploadItem.cpp 2818 2008-03-15 09:38:25Z svn $");
 
 #define AS_BCOMMENT(pointer)        reinterpret_cast<bbs_comment_t *>(pointer)
 #define UINT16(pointer)             static_cast<apr_uint16_t>(pointer)
@@ -239,7 +239,7 @@ void check_item(const char *key, size_t value, size_t expected)
     show_item(key, value);
 }
 
-void run_size()
+void run_size_32()
 {
     show_test_name("size");
 
@@ -266,6 +266,42 @@ void run_size()
     check_item("- comment", OFFSETOF(UploadItem::header_t, comment), 344);
 }
 
+void run_size_64()
+{
+    show_test_name("size");
+
+    check_item("sizeof(UploadItem)", sizeof(UploadItem), 616);
+
+    show_spacer();
+
+    check_item("sizeof(header_t)", sizeof(UploadItem::header_t), 616);
+
+    check_item("- identifier", OFFSETOF(UploadItem::header_t, identifier), 0);
+    check_item("- version", OFFSETOF(UploadItem::header_t, version), 16);
+    check_item("- id", OFFSETOF(UploadItem::header_t, id), 24);
+    check_item("- index", OFFSETOF(UploadItem::header_t, index), 32);
+    check_item("- download_count", OFFSETOF(UploadItem::header_t, download_count), 40);
+    check_item("- file_size", OFFSETOF(UploadItem::header_t, file_size), 48);
+    check_item("- ip_address", OFFSETOF(UploadItem::header_t, ip_address), 104);
+    check_item("- mtime", OFFSETOF(UploadItem::header_t, mtime), 56);
+    check_item("- date", OFFSETOF(UploadItem::header_t, date), 72);
+    check_item("- file_name", OFFSETOF(UploadItem::header_t, file_name), 144);
+    check_item("- file_mime", OFFSETOF(UploadItem::header_t, file_mime), 208);
+    check_item("- file_digest", OFFSETOF(UploadItem::header_t, file_digest), 280);
+    check_item("- remove_pass", OFFSETOF(UploadItem::header_t, remove_pass), 328);
+    check_item("- download_pass", OFFSETOF(UploadItem::header_t, download_pass), 344);
+    check_item("- comment", OFFSETOF(UploadItem::header_t, comment), 360);
+}
+
+void run_size()
+{
+    if (sizeof(long) == 4) {
+        run_size_32();
+    } else {
+        run_size_64();
+    }
+}
+
 void run_all(apr_pool_t *pool, int argc, const char * const *argv)
 {
     if (argc != 1) {
