@@ -32,7 +32,7 @@ use Expect;
 use Term::ReadLine;
 
 use constant CMD_TIMEOUT_SEC    => 10;
-use constant MAKE_TIMEOUT_SEC   => 3600;
+use constant MAKE_TIMEOUT_SEC   => 1200000;
 use constant MOD_UPLOADER_PATH  => 'prog/Apache/Uploader2';
 
 sub create_exp {
@@ -45,7 +45,7 @@ sub ssh_login {
     my $exp = shift;
     my $host = shift;
 
-    $exp->spawn('cocot', qw(-t EUC-JP -p UTF-8 ssh), $host) or die $!;
+    $exp->spawn('ssh', $host) or die $!;
     $exp->expect(CMD_TIMEOUT_SEC, $host . '%');
 
     $exp->send('export LANG=ja_JP.eucJP' . "\n");
@@ -86,7 +86,11 @@ sub make_check {
     $exp->send('make WITH_COLOR=1' . "\n");
     $exp->expect(MAKE_TIMEOUT_SEC, $host . '%');
 
-    $exp->send('sudo make -f GNUmakefile.apache install WITH_COLOR=1' . "\n");
+    $exp->send('sudo su' . "\n");
+    $exp->expect(MAKE_TIMEOUT_SEC, $host . '#');
+    $exp->send('make -f GNUmakefile.apache install WITH_COLOR=1' . "\n");
+    $exp->expect(MAKE_TIMEOUT_SEC, $host . '#');
+    $exp->send('exit' . "\n");
     $exp->expect(MAKE_TIMEOUT_SEC, $host . '%');
 
     $exp->send('make -C src clean-obj WITH_COLOR=1' . "\n");
