@@ -5,6 +5,9 @@ unsetopt FUNCTION_ARGZERO
 PACKAGE=`grep AC_INIT configure.in | perl -pe 's/^[^\(]+[^\[]+\[([^\]]+)\].*$/\1/'`
 VERSION=`grep AC_INIT configure.in | perl -pe 's/^[^,]+\D+([\d.]+)\D.*$/\1/' | tr '.' '_'`
 
+WEB_BASE=/var/www/acapulco.dyndns.org/${PACKAGE}
+CVS_BASE=${HOME}/cvs/${PACKAGE}
+
 echo -n 'windows pass: '
 read -s WIN_PASS
 echo
@@ -20,25 +23,29 @@ run() {
     $@ || exit -1
 }
 
+run ./util/rpm_build.pl
+run ./util/deb_build.pl
 run ./util/win_build.pl
 
 run make -f GNUmakefile.dist dist-sf
-run make -f GNUmakefile.dist dist-win-sf
+run make -f GNUmakefile.dist dist-sf-rpm
+run make -f GNUmakefile.dist dist-sf-deb
+run make -f GNUmakefile.dist dist-sf-win
 
-run cp src/mod_uploader-ap2.0.so /home/apache/${PACKAGE}/
-run cp src/mod_uploader-ap2.2.so /home/apache/${PACKAGE}/
+run cp src/mod_uploader-ap2.0.so ${WEB_BASE}/file
+run cp src/mod_uploader-ap2.2.so ${WEB_BASE}/file
 
 (
     run cd doc
     run make doc-update
     run make doc-api
-    run cp apache.htm apache-win.htm /home/apache/${PACKAGE}/
-    run cp -r api /home/apache/${PACKAGE}/
+    run cp apache.htm apache-win.htm ${WEB_BASE}/file
+    run cp -r api ${WEB_BASE}/file
 )
 
 (
-    run rsync -av --max-size=400000 . ~/cvs/download/${PACKAGE}
-    run cd ~/cvs/download/${PACKAGE}
+    run rsync -av --max-size=400000 . ${CVS_BASE}
+    run cd ${CVS_BASE}
     cvs commit -m ""
     run cvs tag release-${VERSION}_`date "+%y%m%d%H%M"`
 )
